#ifndef __Mail4c_Message_h__
#define __Mail4c_Message_h__

#include <Mail4c/Mail4c.h>

#include <Mail4c/Exception.h>

#include <Base4c/Mutex.h>
#include <Base4c/Semaphore.h>

#include <string>
#include <vector>
#include <map>

class XMLNode;

namespace Mail4c
{
	class Mail4c_Export Message
	{
		public:
			class Mail4c_Export Key
			{
				private:
					std::string _strMessage;

				public:
					Key();
					Key( const std::string& strMessage );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );

					bool operator<( const Key& key ) const;
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;

					const std::string& message() const;
			};

			class Mail4c_Export Party
			{
				public:
					enum Disposition{ dispFrom, dispTo, dispCc, dispBcc };

				private:
					Disposition _disposition;
					std::string _strAddress;

				public:
					Party();
					Party( const std::string& strAddress, Disposition disp = dispFrom );
					Party( const Party& party );
					virtual ~Party();

					const Party& operator=( const Party& party );

					const std::string& address() const;
					Disposition disposition() const;

					static std::string dispositionToText( Disposition disposition );
					static Disposition dispositionFromText( const std::string& strDisposition );
			};

			class Mail4c_Export PartyList : public std::vector<Party>
			{
			};

			class Mail4c_Export Body
			{
				public:
					enum MimeType { typePlain, typeHtml };

				private:
					MimeType _type;
					std::string _strBody;

				public:
					Body();
					Body( MimeType type, const std::string& strBody );
					Body( const Body& body );
					virtual ~Body();

					const Body& operator=( const Body& body );

					MimeType mimeType() const;
					const std::string& body() const;

					static std::string mimeTypeToText( MimeType type );
					static MimeType mimeTypeFromText( const std::string& strMimeType );
			};

			class Mail4c_Export BodyMap : public std::map<Body::MimeType, Body>
			{
			};

			enum Status{ statusPending, statusSent, statusFailed };

		private:
			Key _key;
			Status _status;
			Party _partyFrom;
			PartyList _partyS;
			std::string _strSubject;
			BodyMap _bodyS;

		public:
			Message();
			Message( const Key& key );
			Message( const Message& message );
			virtual ~Message();

			const Message& operator=( const Message& message );

			const Key& key() const;
			Status status() const;
			const Party& from() const;
			const PartyList& recipients() const;
			const std::string& subject() const;
			const BodyMap& bodies() const;

			void setFrom( const Party& party );
			void addRecipient( const Party& party );
			void setSubject( const std::string& strSubject );
			void setBody( const Body& body );

			static std::string statusToText( Status status );
			static Status statusFromText( const std::string& strStatus );
	};
};

#endif /* !__Mail4c_Message_h__ */
